\input LiEwebmac
\noinx

\def\pn{p^n}

@ This file defines just one function, |factor|, which is not very much like
other \LiE. functions, as it produces its result by directly writing to the
output, rather than returning a value.

@h "lie.h"
@f pn NULL

@ The factorisation algorithm is quite trivial. After taking the absolute
value of the argument, we do trial divisions by |2|,~|3|, |5|, and all
numbers relatively prime to |30| and less than $2^{15}$ (the latter
restriction limits the computation time when there are huge prime factors,
and ensures that the relatively fast routine |div1| for division by a single
|digit| can be used). We do not even test whether the square of the trial
factor~|p| exceeds the number~|num| remaining to be factored, but we do stop
when |num| becomes~|1|. If we did not succeed in reducing |num| to~|1|, then
we warn that the final number may not be prime, but only if it exceeds
$2^{30}$, i.e., if it has at least |3|~|digit|s.

@d trial_limit 32768U /* should be between |MaxDigit| and |USHRT_MAX-6| */
@c
static digit inc[] = { 1, 2, 2, 4, 2, 4, 2, 4, 6, 2, 6 };
  /* differences between members of $(\Zed/30)^\ast$ */

object Factor(bigint* num)
{ num=copybigint(num,NULL); @+
  if (num->size<0) {@; Printf("- "); num->size=-num->size; }
  { bigint* temp=mkbigint(num->size); digit p; int i=0;
    if (num->size==0) {@; Printf("0"); goto quit; }
    for (p=2; p<=trial_limit; p+= inc[i++])
    { if (i==array_size(inc)) i=3; /* after |37-31| wrap to difference |11-7| */
      @< Report any factors |p| present in |num| and divide them out;
	 if |num| becomes~|1|, |goto quit| @>
    }
    printbigint(num,0); @+
    if (num->size>2) Printf(" (Last factor need not be a prime)");
  quit:  Printf("\n");
    freemem(num); freemem(temp);
  }
  return (object) NULL;
}

@ Since we are using the destructive routine |div1|, we must always make
sure to duplicate its argument, for in case the division is not exact.
Most trial factors will not divide |num|, so we start with a trial
division to quickly eliminate those factors. The factors~|p| that do
divide~|num|, are likely to be small, and the multiplicity of~|p| might be
high. To speed up that case, we compute the highest power of~|p| that is
acceptable to~|div1|, and divide that out as often as possible before
attempting division by~|p|.

@< Report any factors |p|... @>=
if (copybigint(num,temp),div1(temp,p)==0)
{ index n; digit pn=p; int e=1; @+ copybigint(temp,num);
  for (n=1; pn<=MaxDigit/p; ++n) pn*=p; /* highest $p^n$ fitting in |digit| */
  for (; div1(temp,pn)==0; e+=n) copybigint(temp,num);
    /* find factors $p^n$ */
  if (n>1) /* then there might be some factors |p| left */
    for (copybigint(num,temp); div1(temp,p)==0; ++e) copybigint(temp,num);
      /* factors |p| */
  Printf("%ld",(long)p); @+ if (e>1) Printf("^%ld",(long)e);
  if (cmp1(num,1)==0) goto quit; /* last factor was found */
  Printf(" * ");
}
