\input LiEwebmac

@* Initialisation routines.
This file deals with the functions which will be called at the start
of the program. We write a header file \.{init.h}

@h "lie.h"

@( init.h @>=
@< Prototypes of globally visible variables @>
@< Prototypes of globally visible functions @>

@
@< Prototypes of globally visible functions @>=
void init(void);
boolean Mat_null(object a); 
boolean Vec_null(object a); 
boolean Pol_null(object a); 
poly *poly_one(index r); 
poly *poly_null(index r); 

@
@< Prototypes of globally visible variables @>=
extern object defaultgrp, grp; 
extern bigint* one,* null,* minus_one; 
extern intcel* bool_false,* bool_true;
extern object nothing;

@ These are the constituents of the compilation unit \.{init}.
@c 
@< Global variables @>
@< External variables and functions @>
@< Auxiliary functions @>
@< Init function @>

@
@< Global variables @>=
object defaultgrp = NULL, grp; 
bigint* one,* null,* minus_one; 
intcel* bool_false,* bool_true;
objcel the_nothing,* nothing=&the_nothing;

@
@< External variables and functions @>=

@ These functions have the task to create a series of |NPOLY| 
null-polynomials and a series |NPOLY| of one-polynomials with the
rank varying from~0 to |NPOLY-1|.

@< Aux... @>=
static poly* create_basic_poly(poly **nulls, index r)
{
    int i;
    static boolean first = true;
    poly *result = (poly*) NULL;
    if (first) 
      {@; for (i=0; i<NPOLY; i++) nulls[i]= (poly*) NULL; first = false; }
    if (r<NPOLY && nulls[r]!=NULL) return nulls[r];
    result = mkpoly(1,r); setsorted(result);
    for (i=0; i<r; i++) result->elm[0][i] = 0;
    if (r < NPOLY) {@; nulls[r]=result; setlonglife(result); }
    return result;
}

poly *poly_one(index r)
{
    static poly *ones[NPOLY];
    poly* result=create_basic_poly(ones, r);
    result->coef[0] = one;
    return result;
}

poly *poly_null(index r) 
{
    static poly *zeros[NPOLY];
    poly* result= create_basic_poly(zeros, r);
    result->coef[0] = null;
    return result;
}

@ Here we perform a sequence of actions to prepare for the real work.
@< Init function @>=
void init(void)
{ int ch;
  info_depth.n = 0;
  @< Initialise file names @>
  @< Initialise identifier table @>
  @< Initialise symbol table @>
  @< Allocate memory for objects and programs @>
  @< Define some permanent objects @>
  @< Create top of stack @>
  init_state();
  initialize_readline();
  @< Print a welcoming message @>
  if (enter_input_file(initfil)) ++lineno;
  { char* p=getenv("EDITOR"); @+
      strcpy(editor, p==NULL ? DEFAULT_EDITOR : p);
    @/p=getenv("PAGER");
      strcpy(pager, p==NULL ? DEFAULT_PAGER : p);
   }
   strcpy(promptlabel,PROMPT);
}

@
@d INITFIL "initfile"
@< Initialise file names @>=
{ if (initfil[0]=='\0') strcpy(initfil, INITFIL);
  strcpy(dirpath, infofil);
  strcpy(learnfil, infofil);
  strcpy(titlefil, infofil);
  strcat(infofil,"INFO");
  strcat(learnfil,"LEARN");
  strcat(titlefil,"title");
@/strcpy(infoind, infofil);
  strcpy(learnind, learnfil);
  strcat(infoind,".ind");
  strcat(learnind,".ind");
  strcpy(monfil, "monfile");
}

@
@< Initialise identifier table @>=
{ init_scanner(); /* must store keywords first */
  seq_name=match(".sequence",false);
  if_name=match(".if",false);
  assign_name=match(".assign",false);
  assign_loc_name=match(".assign_loc",false);
  dollar_name=match("$",false);
  break_name=match("_break",false);
  block_name=match("$block",false);
  return_name=match("_return",false);
  setdefault_name=match("_setdefault",false);
  input_fname=name_tab[match("stdin",false)];
}

@ The function |makelink| installs all built-in functions. It uses a
slightly dirty and not fully portable trick to convert \foreign{in situ} the
statically initialised |Symbrec| structures into dynamically computed
|symbrec| structures. These have a different composition, but are intended
to have the same size for all their components (otherwise the conversion
will fail hopelessly). The differences are that the fields |a|, |data|,
|arglst|, and~|next| of |symblst|), with respective types |atp|, |union
{|\dots|}|, |symblst| and~|symblst|, are replaced by fields |name|, |f|,
|arglst|, and~|next|, respective types |(char*)|, |fobject|, |long|,
and~|long|. The string in the |name| field is converted by |match| to an
index in |name_tab| which is stored in the |name| variant of the |a| field
(though this overwrites the pointer to the string, the string itself remains
intact, so |match| does not need to copy the string). The |f| field is
copied to the variant |f| of the |data| field (this statement actually has
no effect). The |long| values in the |arglst| and |next| fields are
relative offsets to following |Symbrec| structures in the same array, and
they are replaced by pointer values computed from them.

@< Auxiliary functions @>=
static void makelink(Symblst a, int n)
{ int i; symblst new_a=(symblst)a; /* overlapping in memory */
  for (i=0; i<n; ++i)
  { if (a[i].name!=NULL) new_a[i].a.name=match(a[i].name,false);
    new_a[i].data.f=a[i].f;
    if (a[i].arglst!=0) new_a[i].arglst=&new_a[i+a[i].arglst];
    else new_a[i].arglst=NULL;
    if (a[i].next!=0) new_a[i].next=&new_a[i+a[i].next];
    else new_a[i].next=NULL;
    }
}

@
@< Initialise symbol table @>=
{ makelink(static1, nstatic1);
  makelink(static2, nstatic2);
  makelink(static3, nstatic3);
  makelink(static4, nstatic4);
  makelink(static5, nstatic5);
  makelink(static6, nstatic6);
  makelink(static7, nstatic7);
}

@
@< Create top of stack @>=
{ top_definitions=creatsym(match(".top_definitions",false));
  add_user_defined();
  topsym=creatsym(match(".topsym",false));
  topsym->formal=match(".topsym_formal",false);
}

@ Here we must call |initmem| before |inittree|, as the latter calls on the
memory allocation functions.

@< Allocate memory for objects and programs @>=
{ maxptrs = MAXPTRS_DFLT; initmem();
  maxnodes = MAXNODES_DFLT; inittree();
}

@
@< Define some permanent objects @>=
{ one=entry2bigint(1);	      setlonglife(one);
  null=entry2bigint(0);	      setlonglife(null);
  minus_one=entry2bigint(-1); setlonglife(minus_one);
@/bool_false=mkintcel(0);     setlonglife(bool_false);
  bool_true=mkintcel(1);      setlonglife(bool_true);
  nothing->any.type=VOID;     setlonglife(nothing);
}

@ We print a sign-on message, unless standard input is redirected; it would
make little sense in that case, and restricting output to answers only makes
\LiE. useful in a pipeline.

@< Print a welcoming message @>=
{ if (!redirected_input)
  { printf("\nLiE version 2.2.2 created on %s\n" @+
      "Authors: Arjeh M. Cohen, Marc van Leeuwen, Bert Lisser.\n", date);
    { FILE* f= fopen(titlefil,"r");
      if (f==NULL) printf("Purpose: development CWI\n\n");
      else {@; while ((ch=getc(f)) != EOF) putchar(ch); fclose(f); }
    }
    printf("\ntype '?help' for help information\n" @+
	   "type '?' for a list of help entries.\n");
  }
}

@
@< Aux... @>=
static boolean eqrow_null(entry *a,index n) 
{
    index i=0;
    while (i<n && *a++ == 0) i++;
    return i==n;
}

boolean Mat_null(object a) 
{
    index    i=0,
    m=a->m.nrows,
    n=a->m.ncols;
    while (i < m && eqrow_null(*(a->m.elm+i),n))
	i++;
    if (i == m) return true;
    return false;
}

boolean Vec_null(object a) 
{
    index	      i, n;
    n=a->v.ncomp;
    i=0;
    while (i < n && *(a->v.compon + i) == 0L)
	i++;
    if (i == n) return true;
    return false;
}

boolean Pol_null(object a) 
{
    index nrows=a->pl.nrows;
    index i=0;
    while (i <nrows && !a->pl.coef[i]->size) i++;
    if (i == nrows) return true;
    return false;
}

@* Index.

