\input LiEwebmac

@* The main program.

This part contains the loop which parses the input string. 
There are a lot of global variable definitions in it.

@< Prototypes of globally visible functions @>=
void allow_intr(void); 
void ignore_intr(void); 

@
@< Prototypes of globally visible variables @>=

extern FILE* cur_in,* cur_out;
extern FILE		*monfile;
extern char		buffer[];
extern char		label2[],monfil[],
infofil[], infoind[], initfil[], learnind[], learnfil[], titlefil[], dirpath[],
promptlabel[], pager[],editor[];
extern int	       lmargin;
extern boolean	       stop;
@)
extern labeltp label;
extern strtype fun_name;
@)
extern char* input_fname;
@)
extern int		block_depth;
extern object		repair_obj;
extern symblst		topsym, top_definitions;
extern jmp_buf		envbuf;
@)
extern boolean
  parsing, isargument, alloc_gc, redirected_input;
extern boolean
  am_monitor, prompt, runtime, verbose, lprint, gc_set, bigint_set;
@)
extern object stop_loop, stop_fun;
@)
extern strtype seq_name, if_name, dollar_name, break_name, return_name,
assign_name, assign_loc_name, block_name,setdefault_name;

@
@( main.h @>=
@< Prototypes of globally visible functions @>
@< Prototypes of globally visible variables @>
@< Externals @>

@
@h "lie.h"

@< Externals @>=
extern int yyparse(void);
#ifdef dos
extern long   farcoreleft(void);
#endif
extern void	add_user_defined();

@ For many of the variables below one might ask why they are defined globally.
Here are the answers in so far as we understand them ourselves. For
|repair_obj|, which is used in assignment operators (see \.{static4.c}) to
hold the variable that is going to be updated, it is necessary because its
reference count has to be fixed by |error| (defined in \.{output.c}) in case
an error occurs in evaluating the right hand side.

@< Definitions of global variables @>=
FILE* cur_in,* cur_out;
FILE		*monfile = NULL;
boolean		stop=false;
char		buffer[LINELENGTH];
char		label2[LABELLENGTH];
char		monfil[LABELLENGTH];
char		infofil[LABELLENGTH];
char		infoind[LABELLENGTH];
char		initfil[LABELLENGTH];
char		learnind[LABELLENGTH];
char		learnfil[LABELLENGTH];
char		titlefil[LABELLENGTH];
char		dirpath[LABELLENGTH];
char		promptlabel[LABELLENGTH];
char		pager[LABELLENGTH],editor[LABELLENGTH];
int		lmargin = 0;
@)
labeltp label;
strtype fun_name;
@)
char* input_fname;
@)
int		block_depth = 0;
object		repair_obj = (object) NULL ;
symblst		topsym, top_definitions;
jmp_buf		envbuf;
@)
boolean
  parsing=true, isargument=false, alloc_gc=true,
  redirected_input=false;
boolean
  am_monitor=false, prompt=true, runtime=false, verbose=false,
  lprint=true, gc_set=true, bigint_set=true;
@)
object stop_loop, stop_fun;
@)
strtype seq_name, if_name, dollar_name, break_name, return_name,
assign_name, assign_loc_name, block_name,setdefault_name;

@
@c
@< Static variables definitions @>
@< Definitions of global variables @>

@
@< Static variables definitions @>=
static boolean	ignore=true; 

@ Here are some simple routines that handle the interrupt signal. We allow
them to be disabled, for instance during critcal memory managemnet
operations. Note however that a signal arriving at such times is simply
discarded; it would be better to block the signal until quiter times and
process it then.

@c
void ignore_intr(void) @+{@; ignore=true; }
void allow_intr(void) @+{@; ignore=false; }

static void handle(int sig)
{
    if (ignore)
	return;
    while (cur_in != stdin) exit_input_file(false); /* pop input files */
    parsing=false;
    error("\nCalculation aborted\n");
}

@ Initialize variables in the main loop.	
@c
static void init_loop(void)
{
    topsym ->next=(symblst) NULL;
    stop_loop=NULL;
    stop_fun=NULL;
    parsing=true;

    tree_pt=0;
    label_pt=0;
    block_depth=0;
    fun_name=0;
    label=label_null;
    if (gc_set)	gc();
    initpar();
    if (cur_out != stdout) cur_out=stdout;
    strcpy(promptlabel,PROMPT);
    fflush(cur_out);
    init_command(); /* this also prompts and calls |inputline| */
    registrate_cpu();
}

@ The main program.
@c
int main(int argc, char** argv)
{ if (no_terminal(stdin)) redirected_input=true;
  cur_in=stdin; cur_out=stdout; /* these cannot be initialised statically */

  @< Read program arguments @>

  init();

    ignore_intr();
    (void) signal(SIGINT, handle);
    (void) setjmp(envbuf);

 /* Finished  signal handling and environment set  */

    while (init_loop(),init_loop_gettype(),yyparse(),!stop)
    {	if (feof(cur_in))
	  stop=exit_input_file(false); /* close file when fully read */
	if (runtime) print_runtime();
    }
    if (!redirected_input) Printf("end program\n");
    return 0;
}

@
@< Read program arguments @>=
{ int nr=1; @+ argc--; infofil[0]='\0';
  if (argc > 0)
  {@;
      strcpy(initfil,argv[nr++]);
      argc--;
  }
  if (argc > 0)
  {@;
      strcpy(infofil,argv[nr++]);
      strcat(infofil,"/");
      argc--;
  }
  if (argc!=0)
  {@;
      printf("Illegal number of arguments.\n");
      exit(1);
  }
}

@* Index.

