This is a change file to mem.w for removing the functions related to
|where_objects|, that are unwanted in a production version.

@x In |findaddr|
  printobjectinfo((object) p);
@y
@z

@x @< Common fields @>=
objtype type; reftype nref; int line; char *fname;
@y
objtype type; reftype nref;
@z

@x
@ Proper objects, like integers, are created by special function calls such
as |mkintcel|. For debugging purposes, we wish to dynamically record the
instance of such a call by which each object was created. To that end, we
define a pair of extra arguments, whose names are always |line| and |file|,
and whose actual arguments are given by the special macros |__LINE__| and
|__FILE__|, which expand to the line number and file name at the point of
call. To avoid having to write these explicitly all the time, we define
functions like |mkintcel| also as macros, whose expansion calls the function
(remember that macros are never recursively invoked in~\Cee) with the
indicated extra arguments. As a consequence we must parenthesise these
function names when they occur in a declaration or heading of the function
definition, in order to prevent macro expansion in these occurrences. Also,
we shall produce declarations of the formal arguments |line| and |file| via
a macro |with_line_and_file|, so that they can be supressed by changing the
definition of this macro; the definitions of the other macros below should
then be changed accordingly. Inside the function definition we invoke
|set_common_fields(x,type)| to initialise the fields common to all object
types; among other things this stores away the |line| and |file| arguments.

@d set_common_fields(x,t)
      ((x)->type=t,(x)->nref=0,(x)->line=line,(x)->fname=file)

@< Public macro... @>=
#define @! with_line_and_file @[@], int line, char* file

#define @! mkintcel(a) mkintcel(a, __LINE__, __FILE__)
#define @! mkbigint(a) mkbigint(a, __LINE__, __FILE__)
#define @! mkvector(a) mkvector(a, __LINE__, __FILE__)
#define @! mkmatrix(a,b) mkmatrix(a,b, __LINE__, __FILE__)
#define @! mkpoly(a,b) mkpoly(a,b, __LINE__, __FILE__)
#define @! mkgroup(a) mkgroup(a, __LINE__, __FILE__)
#define @! mksimpgrp(a,b) mksimpgrp(a,b, __LINE__, __FILE__)
#define @! mktekst(a)  mktekst(a, __LINE__, __FILE__)

@y
@ Proper objects, like integers, are created by special function calls such
as |mkintcel|. In the original program we dynamically record the instance of
such a call by which each object was created using a pair
of extra parameters to functions like |mkintcel|. The change files removes
these parameters by defining the macro |with_line_and_file| used for their
declaration to be empty. Correspondingly |set_common_fields|, which is
invoked in function like |mkintcel|, only sets the |type| and |nref| fields.
The macros defined in the original program for attaching actual arguments
|__LINE__| and |__FILE__| to calls of |mkintcel| and its relatives are not
needed here, so their definition is removed.

@d set_common_fields(x,t) ((x)->type=t,(x)->nref=0)

@< Public macro... @>=
#define @! with_line_and_file /* nothing */

@z

@x
void printobjectinfo(object obj)
{ if (obj!=NULL)
  { string s=type_tag(obj);
    if (s==NULL) Printf("weird object with type %x.\n",type_of(obj));
    else if (type_of(obj)==VOID) Printf("weird VOID object.\n");
    else
      Printf("object defined in line %d of file %s. Type: %s, #refs=%d\n"
	    ,obj->i.line,obj->i.fname,s,refcount(obj));
  }
  return;
}
@y
void printobjectinfo(object obj) {}
@z

@x
@ This function is called when we want to free an object that was not shared.

@c
void share_error(object o)
{ printobjectinfo(o);
@y
@ This function is called when we want to free an object that was not shared.
It provides a hook for inserting debugging code.

@c
void share_error(object o)
{@;
@z

